@@ -2,8 +2,8 @@ package POE::Component::IRC::Plugin::BotCommand;
 BEGIN {
   $POE::Component::IRC::Plugin::BotCommand::AUTHORITY = 'cpan:HINRIK';
 }
-BEGIN {
-  $POE::Component::IRC::Plugin::BotCommand::VERSION = '6.78';
+{
+  $POE::Component::IRC::Plugin::BotCommand::VERSION = '6.79';
 }
 
 use strict;
@@ -20,6 +20,13 @@ sub new {
     $args{Method} = 'notice' if !defined $args{Method};
 
     for my $cmd (keys %{ $args{Commands} }) {
+        if (ref $args{Commands}->{$cmd} eq 'HASH') {
+            croak "$cmd: no info provided" 
+                if !exists $args{Commands}->{$cmd}->{info} ;
+            croak "$cmd: no arguments provided" 
+                if !@{ $args{Commands}->{$cmd}->{args} };
+            
+        }
         $args{Commands}->{lc $cmd} = delete $args{Commands}->{$cmd};
     }
     return bless \%args, $package;
@@ -105,6 +112,47 @@ sub _handle_cmd {
     my $public = $where =~ /^[$chantypes]/ ? 1 : 0;
     $cmd = lc $cmd;
 
+    if (defined $self->{Commands}->{$cmd}) {
+        if (ref $self->{Commands}->{$cmd} eq 'HASH') {
+            my @args_array = defined $args ? split /\s+/, $args : ();      
+            if (@args_array < @{ $self->{Commands}->{$cmd}->{args} } ||
+               (!defined $self->{Commands}->{$cmd}->{variable} && 
+                @args_array > @{ $self->{Commands}->{$cmd}->{args} })
+            ) {
+                  $irc->yield($self->{Method}, $where, 
+                      "Not enough or too many arguments. See help for $cmd");
+                  return;
+            }
+
+            $args = {};
+            for (@{ $self->{Commands}->{$cmd}->{args} }) {
+                my $in_arg = shift @args_array;
+                if (ref $self->{Commands}->{$cmd}->{$_} eq 'ARRAY') {
+                    my @values = @{ $self->{Commands}->{$cmd}->{$_} };
+                    shift @values;
+
+                    use List::MoreUtils qw(none);
+                    # Check if argument has one of possible values
+                    if (none { $_ eq $in_arg} @values) {
+                      $irc->yield($self->{Method}, $where, 
+                          "$_ can be one of ".join '|', @values);
+                      return;
+                    }
+
+                } 
+                $args->{$_} = $in_arg;
+            }
+
+            # Process remaining arguments if variable is set
+            my $arg_cnt = 0;
+            if (defined $self->{Commands}->{$cmd}->{variable}) {
+                for (@args_array) { 
+                    $args->{"opt".$arg_cnt++} = $_;
+                }
+            }
+        }
+    }
+
     if (ref $self->{Auth_sub} eq 'CODE') {
         my ($authed, $errors) = $self->{Auth_sub}->($self->{irc}, $who, $where, $cmd, $args);
 
@@ -145,7 +193,31 @@ sub _get_help {
     if (defined $args) {
         my $cmd = (split /\s+/, $args, 2)[0];
         if (exists $self->{Commands}->{$cmd}) {
-            @help = split /\015?\012/, $self->{Commands}->{$cmd};
+            if (ref $self->{Commands}->{$cmd} eq 'HASH') {
+                push @help, "Syntax: $cmd ".
+                    (join ' ', @{ $self->{Commands}->{$cmd}->{args} }).
+                    (defined $self->{Commands}->{$cmd}->{variable} ? 
+                        " ..."  : "");
+                push @help, split /\015?\012/, 
+                    "Description: ".$self->{Commands}->{$cmd}->{info};
+                push @help, "Arguments:";
+
+                for my $arg (@{ $self->{Commands}->{$cmd}->{args} }) {
+                    next if not defined $self->{Commands}->{$cmd}->{$arg};
+                    if (ref $self->{Commands}->{$cmd}->{$arg} eq 'ARRAY') {
+                        my @arg_usage = @{$self->{Commands}->{$cmd}->{$arg}};
+                        push @help, "    $arg: ".$arg_usage[0].
+                        " (".(join '|', @arg_usage[1..$#arg_usage]).")"
+                    } 
+                    else {
+                        push @help, "    $arg: ".
+                            $self->{Commands}->{$cmd}->{$arg};
+                    }
+                }
+            } 
+            else {
+                @help = split /\015?\012/, $self->{Commands}->{$cmd};
+            }
         }
         else {
             push @help, "Unknown command: $cmd";
@@ -169,6 +241,11 @@ sub add {
     my ($self, $cmd, $usage) = @_;
     $cmd = lc $cmd;
     return if exists $self->{Commands}->{$cmd};
+
+    if (ref $usage eq 'HASH') {
+        return if !exists $usage->{info} || !@{ $usage->{args} };
+    }
+
     $self->{Commands}->{$cmd} = $usage;
     return 1;
 }
@@ -295,6 +372,22 @@ B<'Commands'>, a hash reference, with your commands as keys, and usage
 information as values. If the usage string contains newlines, the plugin
 will send one message for each line.
 
+If a command's value is a HASH ref like this:
+
+     $irc->plugin_add('BotCommand', POE::Component::IRC::Plugin::BotCommand->new(
+         Commands => {
+             slap   => {
+                info => 'Slap someone',
+                args => [qw(nickname)],
+                nickname => 'nickname to slap'
+             }
+         }
+     ));
+
+The args array reference is than used to validate number of arguments required
+and to name arguments passed to event handler. Help is than generated from
+C<info> and other hash keys which represent arguments (they are optional).
+
 =head3 Accepting commands
 
 B<'In_channels'>, a boolean value indicating whether to accept commands in
@@ -365,8 +458,9 @@ L<POE::Component::IRC|POE::Component::IRC>'s C<plugin_add> method.
 =head2 C<add>
 
 Adds a new command. Takes two arguments, the name of the command, and a string
-containing its usage information. Returns false if the command has already been
-defined, true otherwise.
+or hash reference containing its usage information (see C<new>). Returns false 
+if the command has already been defined or no info or arguments are provided, 
+true otherwise.
 
 =head2 C<remove>
 
@@ -376,7 +470,7 @@ if the command wasn't defined to begin with, true otherwise.
 =head2 C<list>
 
 Takes no arguments. Returns a list of key/value pairs, the keys being the
-command names and the values being the usage strings.
+command names and the values being the usage strings or hash references.
 
 =head1 OUTPUT EVENTS
 
@@ -393,7 +487,8 @@ every time someone issued that command. It receives the following arguments:
 =item * C<ARG1> is the name of the channel in which the command was issued,
 or the sender's nickname if this was a private message.
 
-=item * C<ARG2>: a string of arguments to the command, or undef if there
+=item * C<ARG2>: a string of arguments to the command, or hash reference with
+arguments in case you defined command along with arguments, or undef if there
 were no arguments
 
 =back
